@@ -6,45 +6,33 @@ import warnings
 import subprocess
 
 
-class Datamover:
+class Wrapper:
     """
-    Python wrapper for the zih datamover tools that enable exchanging files between a project space on the cluster and a fileserver share via an export node.
-
-    See also
-    --------
-    .. [0] https://doc.zih.tu-dresden.de/data_transfer/datamover/
+    General python wrapper for cluster command line tools.
     """
 
-    def __init__(
-            self, path_to_exe: str = '/sw/taurus/tools/slurmtools/default/bin/', blocking: bool = True, verbosity: int = 0):
+    def __init__(self, path_to_exe: str, glob_pattern: str,
+                 verbosity: int = 0):
         """
-        Sets up a datamover wrapper.
+        Sets up a wrapper for a set of command line tools matching glob_pattern.
 
         Parameters
         ----------
         path_to_exe : str
-            Path to the datamover executables on the cluster, default: /sw/taurus/tools/slurmtools/default/bin/
-        blocking : bool
-            if set to True (default) the slurm jobs are spawned by srun, which runs in the foreground and we can see the output
-            if set to False , the slurm jobs (except dtls) are spawned by sbatch, which runs the job in the background and we would need to extract the output from its log file
-            Do not change this unless you have a good reason to do so.
-        verbosity : int
-            set the verbosity level.
-            0 (default) : verbosity - all output is captured by subprocess and not printed. Both stdout and stderr can later be extracted from proc by `proc.communicate()` (which waits for the process to finish).
+            Path to the datamover executables on the cluster, example: /usr/bin/
+        glob_pattern : str
+            pattern used to find executables within, example: 'ws_*'
+        verbosity : int, optional
+            set the verbosity level, by default 0. Possible values are:
+            0 : no verbosity - all output is captured by subprocess and not printed. Both stdout and stderr can later be extracted from proc by `proc.communicate()` (which waits for the process to finish).
             1 : some verbosity - only stout is captured by subprocess and can be processed later. Errors are printed immediately (useful if you  want to know what is going on when you use Datamover directly in a jupyter notebook).
             2 : very verbose: all output is printed immediately and nothing is captured by subprocess
         """
         self.path_to_exe = Path(path_to_exe)
-        self.blocking = blocking
         self.verbosity = verbosity
-        dtfiles = self.path_to_exe.glob('dt*')
-        self.exe = [f.name for f in dtfiles if os.access(f, os.X_OK)]
-        if 'dtls' in self.exe:
-            self.current_command = 'dtls'
-        elif len(self.exe) > 0:
-            self.current_command = self.exe[0]
-        else:
-            self.current_command = None
+        exe_files = self.path_to_exe.glob(glob_pattern)
+        self.exe = [f.name for f in exe_files if os.access(f, os.X_OK)]
+        self.current_command = None
 
     def __getattr__(self, attr):
         '''
@@ -75,8 +63,6 @@ class Datamover:
         # Also, dtls behaves the same as all other processes (by default all
         # processes except dtls use sbatch)
         args = [self.path_to_exe / self.current_command] + list(args)
-        if self.blocking:
-            args += ['--blocking']
         stdout = subprocess.PIPE if self.verbosity < 2 else None
         stderr = subprocess.PIPE if self.verbosity < 1 else None
         proc = subprocess.Popen(args, stdout=stdout, stderr=stderr)
@@ -86,8 +72,65 @@ class Datamover:
         return len(self.exe) > 0
 
 
-class Workspace(Datamover):
-    """Wrapper for the workspace management tools on taurus
+class Datamover(Wrapper):
+    """
+    Python wrapper for the zih datamover tools that enable exchanging files between a project space on the cluster and a fileserver share via an export node.
+
+    See also
+    --------
+    .. [0] https://doc.zih.tu-dresden.de/data_transfer/datamover/
+    """
+
+    def __init__(
+            self, path_to_exe: str = '/sw/taurus/tools/slurmtools/default/bin/', blocking: bool = True, verbosity: int = 0):
+        """
+        Sets up a datamover wrapper.
+
+        Parameters
+        ----------
+        path_to_exe : str, optional
+            Path to the datamover executables on the cluster, by default: /sw/taurus/tools/slurmtools/default/bin/
+        blocking : bool, optional
+            if set to True (default) the slurm jobs are spawned by srun, which runs in the foreground and we can see the output
+            if set to False , the slurm jobs (except dtls) are spawned by sbatch, which runs the job in the background and we would need to extract the output from its log file
+            Do not change this unless you have a good reason to do so.
+        verbosity : int, optional
+            set the verbosity level, by default 0. Possible values are:
+            0 : no verbosity - all output is captured by subprocess and not printed. Both stdout and stderr can later be extracted from proc by `proc.communicate()` (which waits for the process to finish).
+            1 : some verbosity - only stout is captured by subprocess and can be processed later. Errors are printed immediately (useful if you  want to know what is going on when you use Datamover directly in a jupyter notebook).
+            2 : very verbose: all output is printed immediately and nothing is captured by subprocess
+        """
+        self.blocking = blocking
+        super().__init__(
+            path_to_exe=path_to_exe,
+            verbosity=verbosity,
+            glob_pattern='dt*')
+
+    def _execute(self, *args):
+        """
+        Execute the current command with arguments and return its output.
+
+        Parameters
+        ----------
+        args : list of str
+            The arguments to the command to be executed, e.g. for the command "dtls" sensible arguments would be ["-l", "/foo/bar"]
+
+        Returns
+        -------
+        subprocess.Popen object (see: https://docs.python.org/3/library/subprocess.html#popen-constructor)
+        """
+        # we append the argument "--blocking" so that datamover uses srun
+        # instead of sbatch for all arguments. That way, we can use
+        # subprocess.poll to figure out whether the operation has finished.
+        # Also, dtls behaves the same as all other processes (by default all
+        # processes except dtls use sbatch)
+        if self.blocking:
+            args += ('--blocking',)
+        return super()._execute(*args)
+
+
+class Workspace(Wrapper):
+    """Python wrapper for the workspace management tools on taurus
 
         See also
         --------
@@ -103,24 +146,16 @@ class Workspace(Datamover):
         ----------
         path_to_exe : str
             Path to the datamover executables on the cluster, default: /usr/bin/
-        verbosity : int
-            set the verbosity level.
-            0 (default) : verbosity - all output is captured by subprocess and not printed. Both stdout and stderr can later be extracted from proc by `proc.communicate()` (which waits for the process to finish).
+        verbosity : int, optional
+            set the verbosity level, by default 0. Possible values are:
+            0 : no verbosity - all output is captured by subprocess and not printed. Both stdout and stderr can later be extracted from proc by `proc.communicate()` (which waits for the process to finish).
             1 : some verbosity - only stout is captured by subprocess and can be processed later. Errors are printed immediately (useful if you  want to know what is going on when you use Datamover directly in a jupyter notebook).
             2 : very verbose: all output is printed immediately and nothing is captured by subprocess
-
         """
-        self.path_to_exe = Path(path_to_exe)
-        self.blocking = False
-        self.verbosity = verbosity
-        ws_files = self.path_to_exe.glob('ws_*')
-        self.exe = [f.name for f in ws_files if os.access(f, os.X_OK)]
-        if 'ws_list' in self.exe:
-            self.current_command = 'ws_list'
-        elif len(self.exe) > 0:
-            self.current_command = self.exe[0]
-        else:
-            self.current_command = None
+        super().__init__(
+            path_to_exe=path_to_exe,
+            verbosity=verbosity,
+            glob_pattern='ws_*')
 
 
 class CacheWorkspace:
@@ -166,7 +201,7 @@ class CacheWorkspace:
 
         Files will be deleted automatically one day after release.
         '''
-        waitfor(self.ws.ws_release(self.id))
+        self.ws.ws_release(self.id).wait()
         self.name = None
 
     def _mk_cache(self) -> str:
@@ -179,7 +214,6 @@ class CacheWorkspace:
         '''
         proc = self.ws.ws_allocate(self.id, str(
             self.expire_in_days), '-c', 'temporary_files')
-        waitfor(proc, discard_output=False)
         out, _ = proc.communicate()
         self.name = out.strip().decode('utf-8')
 
@@ -231,7 +265,7 @@ def save_to_project(save_fn: callable, file_path: str, *args, file_path_arg_pos:
     # datamover tools
     dm = Datamover(path_to_exe=path_to_datamover)
     with tempfile.TemporaryDirectory(prefix=cache_workspace.name + '/') as tmp:
-        temp_file = Path(tmp.name) / full_path.name
+        temp_file = Path(tmp) / full_path.name
         args = args[0:file_path_arg_pos] + \
             (str(temp_file),) + args[file_path_arg_pos:]
         output = save_fn(*args, **kw)
